@@ -6,6 +6,7 @@
 #include "xs_json.h"
 #include "xs_socket.h"
 #include "xs_unix_socket.h"
+#include "xs_http.h"
 #include "xs_httpd.h"
 #include "xs_mime.h"
 #include "xs_time.h"
@@ -59,7 +60,7 @@ static jmp_buf on_break;
 /** code **/
 
 /* nodeinfo 2.0 template */
-const char *nodeinfo_2_0_template = ""
+const char * const nodeinfo_2_0_template = ""
     "{\"version\":\"2.0\","
     "\"software\":{\"name\":\"snac\",\"version\":\"" VERSION "\"},"
     "\"protocols\":[\"activitypub\"],"
@@ -172,7 +173,7 @@ static xs_str *greeting_html(void)
 }
 
 
-const char *share_page = ""
+const char * const share_page = ""
 "<!DOCTYPE html>\n"
 "<html>\n"
 "<head>\n"
@@ -190,7 +191,7 @@ const char *share_page = ""
 "";
 
 
-const char *authorize_interaction_page = ""
+const char * const authorize_interaction_page = ""
 "<!DOCTYPE html>\n"
 "<html>\n"
 "<head>\n"
@@ -597,6 +598,9 @@ void httpd_connection(FILE *f)
     if (status == HTTP_STATUS_NOT_FOUND)
         body = xs_str_new("<h1>404 Not Found (" USER_AGENT ")</h1>");
 
+    if (status == HTTP_STATUS_GONE)
+        body = xs_str_new("<h1>410 Gone (" USER_AGENT ")</h1>");
+
     if (status == HTTP_STATUS_BAD_REQUEST && body != NULL)
         body = xs_str_new("<h1>400 Bad Request (" USER_AGENT ")</h1>");
 
@@ -650,7 +654,7 @@ void httpd_connection(FILE *f)
     if (p_state->use_fcgi)
         xs_fcgi_response(f, status, headers, body, b_size, fcgi_id);
     else
-        xs_httpd_response(f, status, http_status_text(status), headers, body, b_size);
+        xs_httpd_response(f, status, xs_http_status_text(status), headers, body, b_size);
 
     fclose(f);
 
