@@ -10,13 +10,14 @@
 #include "xs_glob.h"
 #include "xs_curl.h"
 #include "xs_regex.h"
+#include "xs_http.h"
 
 #include "snac.h"
 
 #include <sys/stat.h>
 #include <stdlib.h>
 
-static const char *default_srv_config = "{"
+static const char * const default_srv_config = "{"
     "\"host\":                 \"\","
     "\"prefix\":               \"\","
     "\"address\":              \"127.0.0.1\","
@@ -42,7 +43,7 @@ static const char *default_srv_config = "{"
     "\"fastcgi\":              false"
     "}";
 
-static const char *default_css =
+static const char * const default_css =
     "body { max-width: 48em; margin: auto; line-height: 1.5; padding: 0.8em; word-wrap: break-word; }\n"
     "pre { overflow-x: scroll; }\n"
     "blockquote { font-style: italic; }\n"
@@ -97,7 +98,7 @@ const char *snac_blurb =
     "automatic sign-up process.</p>\n"
 ;
 
-static const char *greeting_html =
+static const char * const greeting_html =
     "<!DOCTYPE html>\n"
     "<html><head>\n"
     "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\"/>\n"
@@ -453,6 +454,8 @@ int deluser(snac *user)
         }
     }
 
+    grave(user->uid, 1);
+
     rm_rf(user->basedir);
 
     return ret;
